/*
 * error.c - Error reporting functions
 *
 * Written 2001 by Werner Almesberger
 * Copyright 2001 EPFL-ICA, Network Robots
 */


#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <execinfo.h>

#include "config.h"
#include "error.h"


/* ----- Debugging messages with implict indentation ----------------------- */


static int stack_zero; /* minimum stack depth */


static int backtrace_len(void)
{
    void *dummy[MAX_DEBUG_DEPTH];

    return backtrace(dummy,MAX_DEBUG_DEPTH);
}


static void vdebugf(const char *msg,va_list ap)
{
    int depth,i;

    depth = backtrace_len(); /* auto-calibrate */
    if (!stack_zero || depth < stack_zero) stack_zero = depth;
    for (i = depth-stack_zero; i; i--) putchar(' ');
    vprintf(msg,ap);
    putchar('\n');
}


void debugf(const char *msg,...)
{
    va_list ap;

    if (!debug) return;
    va_start(ap,msg);
    vdebugf(msg,ap);
    va_end(ap);
}


void debugf2(const char *msg,...)
{
    va_list ap;

    if (debug < 2) return;
    va_start(ap,msg);
    vdebugf(msg,ap);
    va_end(ap);
}


/* ----- Fine-grained warning configuration -------------------------------- */


int warn_truncate = 0; /* see config.h for description */
int warn_unused = 1;


int set_warn_switch(const char *arg)
{
    int value = 1;

    if (!strncmp(arg,"no",2)) {
	arg += 2;
	value = 0;
    }
    if (!strcmp(arg,"truncate")) warn_truncate = value;
    else if (!strcmp(arg,"unused")) warn_unused = value;
    else return 0;
    return 1;
}
